import { routerRedux } from 'dva/router';
import { login, userInfo } from '../services/login';
import { validate } from '../services/login/password';
import { fullPath } from '../utils/helper';
import { setToken, setUser, setDomain, histories } from '../utils/auth';
import { switchDomain, currentDomain } from '../services/domain';
import { errors } from '../utils/error';
import config from '../utils/config';

export default {
  namespace: 'login',
  state: {},
  reducers: {},
  effects: {
    *login({ payload }, { call, put }) {
      const loginRequest = {
        type: 'userName',
        data: payload.userName,
        authRequest: yield call(validate, payload.password),
      };
      const result = yield call(login, loginRequest);
      const { tokenId, authResponse, remainedAuthRequirements } = result;
      if (authResponse.status !== 'authed' && authResponse.status !== 'skipped') {
        throw errors.wrongPassword();
      }
      const { requirements } = remainedAuthRequirements;
      if (requirements.length > 0) {
        throw errors.unsupportedAuthType(requirements);
      }
      yield call(setToken, tokenId);
      const uInfo = yield call(userInfo);
      yield call(setUser, uInfo.id, uInfo.name);
      const path = yield call(histories.getLatest, 'domain');
      if (!path) {
        yield put(routerRedux.push(fullPath('/domain')));
      } else {
        yield call(switchDomain, path);
        const domain = yield call(currentDomain);
        if (domain) {
          yield call(setDomain, domain.name, path);
          const latest = yield call(histories.getLatest, 'module');
          if (latest && config.fastNavigationPage) {
            yield put(routerRedux.push(fullPath('/fastNav')));
          } else {
            yield put(routerRedux.push(fullPath('/main')));
          }
        } else {
          yield put(routerRedux.push(fullPath('/domain')));
        }
      }
    },
  },
  subscriptions: {},
};
